// NotificationMonitor.cpp
#include <windows.h>
#include "NotificationMonitor.h"
#include <msgqueue.h>
#include <pnp.h>
#include <pm.h>

GUID  STORAGE_GUID = {0xA4E7EDDA, 0xE575, 0x4252, { 0x9D, 0x6B, 0x41, 0x95, 0xD4, 0x8B, 0xB8, 0x65} };

typedef union {
  DEVDETAIL d;
  char pad[sizeof(DEVDETAIL)+MAX_DEVCLASS_NAMELEN];
} STORAGE_DEVICE_NOTIFY_INFO;

volatile BOOL NotificationMonitor::m_stop;

NotificationMonitor::NotificationMonitor()
{
	MSGQUEUEOPTIONS options;

	// set our global thread stop flag
	m_stop = FALSE;

	// same options will be used for both queues
	options.dwFlags = MSGQUEUE_NOPRECOMMIT;
	options.dwMaxMessages = 0;		// no limit
	options.cbMaxMessage = 2048;	// more than enough space for the messages we're after
	options.bReadAccess = true;		// read queue

	// create our queues
	m_powerQueue = CreateMsgQueue(_T("PowerEventWaitQueue"), &options);
	m_storageQueue = CreateMsgQueue(_T("StorageEventWaitQueue"), &options);

	// create our monitor threads
	m_powerThread = CreateThread(NULL, 0, &PowerMonitorThreadProc, (void*)this, 0, NULL);
	m_storageThread = CreateThread(NULL, 0, &StorageMonitorThreadProc, (void*)this, 0, NULL);

}

NotificationMonitor::~NotificationMonitor()
{
	// shut down the worker threads
	m_stop = TRUE;

	// destroy the queues
	CloseMsgQueue(m_powerQueue);
	CloseMsgQueue(m_storageQueue);
}


DWORD NotificationMonitor::PowerMonitorThreadProc(void *data)
{
	POWER_BROADCAST pb;
	DWORD flags;
	DWORD size;

	NotificationMonitor *instance = (NotificationMonitor*)data;

	HANDLE notificationHandle = RequestPowerNotifications(instance->m_powerQueue, PBT_RESUME);

	while(TRUE)
	{
		// wait for something in the queue
		if(WaitForSingleObject(instance->m_powerQueue, 1000) == WAIT_OBJECT_0)
		{
			// get data from the queue
			while(ReadMsgQueue(instance->m_powerQueue, &pb, sizeof(pb), &size, 0, &flags))
			{
				// this check is superfluous, since we only subscribed to PBT_RESUME, 
				// but for completeness of the sample we do it anyway
				if(pb.Message & PBT_RESUME)
				{
					// call OnPowerResume
					instance->OnPowerResume();
				}
			}
		}


		if(m_stop)
			break;
		
	}

	StopPowerNotifications(instance->m_powerQueue);

	return 0;
}

DWORD NotificationMonitor::StorageMonitorThreadProc(void *data)
{
	STORAGE_DEVICE_NOTIFY_INFO deviceInfo;
	DWORD flags;
	DWORD size;

	NotificationMonitor *instance = (NotificationMonitor*)data;

	// request notifcations
	HANDLE notificationHandle = RequestDeviceNotifications(&STORAGE_GUID, instance->m_storageQueue, TRUE);

	// wait for notifications
	while(TRUE)
	{
		// check back every second to see if we should exit the thread
		if(WaitForSingleObject(instance->m_storageQueue, 1000) == WAIT_OBJECT_0)
		{
			// get data from the queue
			while(ReadMsgQueue(instance->m_storageQueue, &deviceInfo, sizeof(deviceInfo), &size, 1, &flags))
			{
				if(deviceInfo.d.fAttached)
				{
					// call the virtual function
					instance->OnCardInsert();
				}
				else
				{
					// call the virtual function
					instance->OnCardRemove();
				}
			}
		}

		if(m_stop)
			break;
	}

	// stop notifications
	StopDeviceNotifications(notificationHandle);

	return 0;
}


